import os
import shutil
from pathlib import Path

import click
from cicd.core.logger import logger
from cicd.core.utils.sh import sh
from cicd.ios.mixin.build import BuildMixin

__all__ = ['main']


class Demo:
    def __init__(self) -> None:
        self.tmp_dir = Path('/tmp/wikipedia-ios')
        self.repo_url = 'https://github.com/wikimedia/wikipedia-ios.git'

    def build(self):
        self.clone_project()
        self.xcodebuild()
        self.copy_app_bundle()

    def clone_project(self):
        if not self.tmp_dir.exists():
            cmd = f'git clone --depth=1 --single-branch {self.repo_url} {self.tmp_dir}'
            sh.exec(cmd, capture_output=False, log_cmd=True)

    def xcodebuild(self):
        logger.info('Building the project. This might take a while...')
        workdir = Path().absolute()
        os.chdir(self.tmp_dir)
        try:
            BuildMixin(
                scheme='Wikipedia',
                destination='generic/platform=iOS Simulator',
                derived_data_path='DerivedData',
            ).start_building()
        finally:
            os.chdir(workdir)

    def copy_app_bundle(self):
        src_path = (
            self.tmp_dir
            / 'DerivedData'
            / 'Build'
            / 'Products'
            / 'Debug-iphonesimulator'
            / 'Wikipedia.app'
        )
        dst_path = Path('tmp/apps/example.app')
        logger.info(f'Copy app bundle from {src_path} to {dst_path}')
        if dst_path.exists():
            shutil.rmtree(dst_path)
        shutil.copytree(src_path, dst_path)


@click.group()
def main():
    '''CLI to work with the demo.'''


@main.command
def build():
    Demo().build()


if __name__ == '__main__':
    main()
